package com.ruoyi.web.controller.app;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.ss.order.domain.OrderQuery;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.order.domain.dto.OrderDTO;
import com.ruoyi.ss.order.domain.dto.ReOrderDTO;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.order.service.IOrderValidator;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 订单Controller
 * @author 辉
 * 2024/3/5
 */
@Api(tags = "订单")
@RestController
@RequestMapping("/app/order")
public class AppOrderController extends BaseController {

    @Autowired
    private IOrderService orderService;

    @Autowired
    private IOrderValidator orderValidator;

    /**
     * 下单
     */
    @Log(title = "下单", businessType = BusinessType.CREATEORDER)
    @PostMapping("/createOrder")
    public AjaxResult createOrder(@RequestBody @Validated OrderDTO order)
    {
        logger.info("【下单】请求参数：order={}", JSON.toJSON(order));
        PrepayResponseVO orderServiceOrder = orderService.createOrder(order);
        return success(orderServiceOrder);
    }

    /**
     * 订单取消
     */
    @Log(title = "订单取消", businessType = BusinessType.ORDERCANCEL)
    @PostMapping("/orderCancel")
    public AjaxResult orderCancel(String orderNo)
    {
        logger.info("【订单取消】请求参数：orderNo={}", orderNo);
        Boolean aBoolean = orderService.orderCancel(orderNo);
        return toAjax(aBoolean);
    }

    /**
     * 续单
     */
    @Log(title = "续单", businessType = BusinessType.REORDER)
    @PostMapping("/reOrder")
    public AjaxResult reOrder(@RequestBody @Validated ReOrderDTO order)
    {
        logger.info("【续单】请求参数：order={}", JSON.toJSON(order));
        PrepayResponseVO orderServiceOrder = orderService.reOrder(order);
        return success(orderServiceOrder);
    }

    /**
     * 更换包间
     * 1. 订单号、新房间号
     * 2. 校验：
     *      2.1 不能与原房间号相同
     *      2.2 新包间已预定的不能更换
     */
    @Log(title = "更换包间", businessType = BusinessType.REORDER)
    @PostMapping("/changeRoom")
    public AjaxResult changeRoom(OrderQuery order)
    {
        logger.info("【更换包间】请求参数：order={}", JSON.toJSON(order));
        orderValidator.validateChangeRoom(order);
        return toAjax(orderService.changeRoom(order));
    }

    /**
     * 我的订单列表
     */
    @GetMapping("/myOrderList")
    public AjaxResult myOrderList(OrderQuery order)
    {
        logger.info("订单列表请求：OrderQuery=【{}】", JSON.toJSONString(order));
        order.setUserId(getUserId());
        if(order.getStatusList() == null || order.getStatusList().length == 0){
            order.setStatusList(null);
        }
        List<OrderVO> rlOrderVOS = orderService.selectRlOrderListNoIsolate(order);
        return success(rlOrderVOS);
    }


}
