package com.ruoyi.web.controller.app;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.User;
import com.ruoyi.common.core.domain.entity.UserVO;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.verify.vo.IDResponse;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.ss.article.domain.Article;
import com.ruoyi.ss.article.service.IArticleService;
import com.ruoyi.common.core.domain.entity.ArticleClassify;
import com.ruoyi.ss.articleClassify.service.IArticleClassifyService;
import com.ruoyi.ss.changeBalance.domain.ChangeBalance;
import com.ruoyi.ss.changeBalance.domain.ChangeBalanceQuery;
import com.ruoyi.ss.channel.service.IChannelService;
import com.ruoyi.ss.channelWithdraw.domain.ChannelWithdrawQuery;
import com.ruoyi.ss.channelWithdraw.domain.ChannelWithdrawVO;
import com.ruoyi.ss.channelWithdraw.service.ChannelWithdrawService;
import com.ruoyi.ss.feeRule.domain.FeeRule;
import com.ruoyi.ss.feeRule.service.IFeeRuleService;
import com.ruoyi.ss.feedback.domain.Feedback;
import com.ruoyi.ss.feedback.service.IFeedbackService;
import com.ruoyi.ss.model.service.IModelService;
import com.ruoyi.ss.order.domain.OrderQuery;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.orderOper.service.IOrderOperService;
import com.ruoyi.ss.user.service.IUserService;
import com.ruoyi.ss.userWithdraw.service.IUserWithdrawService;
import com.ruoyi.ss.withdraw.domain.WithdrawQuery;
import com.ruoyi.ss.withdraw.service.IWithdrawService;
import com.ruoyi.system.domain.query.AuthenticationQuery;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.IWxPayService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

;

/**
 * app接口（需要登录校验的）
 *  校验
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/appVerify")
public class AppVerifyController extends BaseController
{

    @Resource
    private IUserService userService;

    @Resource
    private IOrderService orderService;

//    @Autowired
//    private IEDeviceService deviceService;

    @Autowired
    private IWithdrawService withdrawService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private IChannelService channelService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IOrderOperService orderOperService;

    @Resource
    private IFeedbackService feedbackService;

    @Autowired
    private IUserWithdrawService userWithdrawService;

    @Autowired
    private ChannelWithdrawService etChannelWithdrawService;

    @Autowired
    private ISysDictDataService dictDataService;

    @Autowired
    private IFeeRuleService feeRuleService;

    @Autowired
    private IModelService modelService;

    @Autowired
    private IArticleService articleService;

    @Autowired
    private IArticleClassifyService articleClassifyService;

    /**
     * 首页统计
     */
    @GetMapping(value = {  "/index" })
    public AjaxResult index(String startTime,String endTime)
    {
        logger.info("【首页统计/index】参数：startTime={},endTime={}", startTime,endTime);
        AjaxResult ajax = AjaxResult.success();
        IndexAdminVo incomeVos = orderService.indexStatistics(startTime, endTime,getUserId());
        ajax.put(AjaxResult.DATA_TAG, incomeVos);
        return ajax;
    }

    /**
     * 实名认证
     */
    @Log(title = "实名认证", businessType = BusinessType.AUTHENTICATION)
    @GetMapping("/user/authentication")
    public AjaxResult authentication(AuthenticationQuery authenticationQuery)
    {
        logger.info("【实名认证】请求参数：authenticationQuery={}", JSON.toJSON(authenticationQuery));
        Object authentication = userService.authentication(authenticationQuery);
        //判断o是否是字符串
        if(authentication instanceof String){
            return AjaxResult.error((String) authentication);
        }else{
            IDResponse.Result result = (IDResponse.Result)authentication;
            if(result.getIsok()){
                //保存身份信息
                // 更新用户并更新缓存
                LoginUser loginUser = SecurityUtils.getLoginUser();
                User asUser = new User();
                asUser.setUserId(authenticationQuery.getUserId());
                asUser.setRealName(authenticationQuery.getRealName());
                asUser.setIdCard(authenticationQuery.getIdCard());
                asUser.setIsAuthentication(true);
                asUser.setPhonenumber(authenticationQuery.getPhone());
                int updateUser = userService.updateUser(asUser);
                if (updateUser>0)
                {
                    logger.info("【实名认证】保存身份信息成功");
                    // 更新缓存用户信息
                    tokenService.setLoginUser(loginUser);
                }
            }
            return AjaxResult.success(authentication);
        }
    }

    /**
     * 确认支付
     */
    @Log(title = "确认支付", businessType = BusinessType.CONFIRMPAY)
    @PostMapping("/confirmPay")
    public AjaxResult confirmPay(String orderNo)
    {
        logger.info("【确认支付】请求参数：orderNo={}",orderNo);
        PrepayResponseVO orderServiceOrder = orderService.confirmPay(orderNo);
        return success(orderServiceOrder);
    }

    /**
     * 订单列表
     */
    @GetMapping("/orderList")
    public AjaxResult orderList(OrderQuery order)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSONString(order)+"】");
        List<OrderVO> rlOrderVOS = orderService.selectRlOrderList(order);
        return success(rlOrderVOS);
    }



    /**
     * 订单信息
     */
    @GetMapping("/orderInfo")
    public AjaxResult orderInfo(String orderNo)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSON(orderNo)+"】");
        OrderVO rlOrder = orderService.getOrderInfoByOrderNo(orderNo);
        return success(rlOrder);
    }

//    /**
//     * 扫码绑定车辆
//     */
//    @Log(title = "扫码绑定车辆", businessType = BusinessType.BANDDEVICE)
//    @PostMapping("/bandDevice")
//    public AjaxResult bandDevice(String orderNo, String sn)
//    {
//        logger.info("扫码绑定车辆：orderNo=【"+orderNo+"】,sn=【"+sn+"】");
//        orderService.bandDevice(orderNo,sn);
//        return success();
//    }

    /**
     * 用户扫码绑定车辆
     */
    @Log(title = "用户扫码绑定车辆", businessType = BusinessType.BANDDEVICE)
    @PostMapping("/userBandDevice")
    public AjaxResult userBandDevice(String sn)
    {
        logger.info("扫码绑定车辆：sn=【"+sn+"】");
        if(StrUtil.isBlank(sn)){
            throw new ServiceException("设备sn不能为空");
        }
        int i = orderService.userBandDevice(getUserId(),sn);
        if(i<=0){
            return AjaxResult.error("绑定失败");
        }
        return success(i);
    }

//    /**
//     * 临时锁车
//     */
//    @Log(title = "临时锁车", businessType = BusinessType.LOCK)
//    @PostMapping("/device/lock")
//    public AjaxResult deviceLock(String sn,String orderNo,String isBluetooth,String lon,String lat)
//    {
//        logger.info("【临时锁车请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
//        Boolean i =deviceService.lock(sn,orderNo,isBluetooth,lon,lat);
//        return success(i);
//    }

//    /**
//     * 临时解锁
//     */
//    @Log(title = "临时解锁", businessType = BusinessType.UNLOCK)
//    @PostMapping("/device/unlock")
//    public AjaxResult deviceUnlock(String sn,String orderNo,String isBluetooth,String lon,String lat)
//    {
//        logger.info("【临时解锁请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
//        if(StrUtil.isBlank(orderNo)){
//            throw new ServiceException("订单号为空");
//        }
//        Boolean i =deviceService.unlock(sn,orderNo,isBluetooth,lon,lat);
//        return success(i);
//    }

//    /**
//     * 修改车辆
//     */
//    @Log(title = "修改车辆", businessType = BusinessType.UPDATE)
//    @PutMapping("/device/edit")
//    public AjaxResult deviceEdit2(@RequestBody EDeviceQuery deviceQuery)
//    {
//        logger.info("【修改车辆】请求参数：device={}", JSON.toJSONString(deviceQuery));
//        if(StrUtil.isBlank(deviceQuery.getSn())){
//            return error("【修改车辆】sn不能为空");
//        }
//        EDevice rlDevice = deviceService.selectEDeviceBySn(deviceQuery.getSn());
//        if(rlDevice == null){
//            return error("【修改车辆】设备不存在【"+deviceQuery.getSn()+"】");
//        }
//        return toAjax(deviceService.updateEDeviceBySn(deviceQuery));
//    }


//    /**
//     * 续单
//     */
//    @Log(title = "续单", businessType = BusinessType.RELET)
//    @PostMapping("/relet")
//    public AjaxResult relet(@RequestBody OrderQuery orderQuery)
//    {
//        logger.info("【续单】请求参数：order={}", JSON.toJSONString(orderQuery));
//        orderService.validate(orderQuery);
//        PrepayResponseVO responseVO =orderService.relet(orderQuery);
//        return success(responseVO);
//    }

//    /**
//     * 还车
//     */
//    @Log(title = "还车", businessType = BusinessType.RETURN_VEHICLE)
//    @PostMapping("/returnVehicle")
//    public AjaxResult returnVehicle(@RequestBody EOrderQuery order)
//    {
//        logger.info("【还车】请求参数：order={}", JSON.toJSONString(order));
//        Boolean aBoolean = deviceService.returnVehicle(order);
//        return success(aBoolean);
//    }


    /**
     * 获取提现记录详细信息
     */
    @GetMapping(value = "/withdraw/{id}")
    public AjaxResult withdrawInfo(@PathVariable("id") Long id)
    {
        return success(withdrawService.selectRlWithdrawById(id));
    }

    /**
     * 我的账户
     */
    @GetMapping(value = {  "/myAccountInfo" })
    public AjaxResult myAccountInfo()
    {
        AjaxResult ajax = AjaxResult.success();
        UserVO rlUser = userService.myAccountInfoByUserId(getUserId());
        ajax.put(AjaxResult.DATA_TAG, rlUser);
        return ajax;
    }

    /**
     * 账变记录
     */
    @GetMapping(value = {  "/myChangeBalanceList" })
    public AjaxResult myChangeBalanceList(ChangeBalanceQuery query)
    {
        logger.info("【账变记录/myChangeBalanceList】参数：【query={}】", query);
        AjaxResult ajax = AjaxResult.success();
        if(ObjectUtil.isNotNull(query.getType()) && query.getType().equals("0")){
            query.setType(null);
        }
        List<ChangeBalance> changeBalances  = userService.myChangeBalanceList(query,getUserId());
        ajax.put(AjaxResult.DATA_TAG, changeBalances);
        return ajax;
    }


//    /**
//     * 改价
//     */
//    @Log(title = "订单改价", businessType = BusinessType.EDITPRICE)
//    @PutMapping("/order/changePrice")
//    public AjaxResult orderEdit(@RequestBody EOrderQuery order)
//    {
//        logger.info("【订单改价信息】：{}", JSON.toJSON(order));
//        //根据订单号查询订单信息
//        EOrder etOrder1 = orderService.selectRlOrderByOrderNo(order.getOrderNo());
//        if(ObjectUtil.isNull(etOrder1)){
//            throw new ServiceException("订单不存在");
//        }
//        if(ServiceConstants.ORDER_PAY_STATUS_PAID.equals(etOrder1.getPaid())){
//            throw new ServiceException("订单已支付，不能改价");
//        }
//        if(!ServiceConstants.ORDER_STATUS_TO_BE_PAID.equals(etOrder1.getStatus())){
//            throw new ServiceException("非待支付订单无法改价");
//        }
//        String outTradeNo = etOrder1.getOutTradeNo();
//        ChannelVO channelVO = channelService.selectSmChannelByChannelId(etOrder1.getPayChannel());
//        Boolean execute = transactionTemplate.execute(e -> {
//            if(StrUtil.isNotBlank(outTradeNo)){
//                /** 关闭未支付订单 */
//                if(wxPayService.closeOrder(outTradeNo,channelVO)){
//                    throw new ServiceException("关闭未支付订单失败");
//                }
//                logger.info("【订单改价】订单【{}】,有outTradeNo = 【{}】,查询订单未支付，关闭订单：{}", etOrder1.getOrderNo(),outTradeNo);
//                BigDecimal payFee = BigDecimal.ZERO;
//                if(ObjectUtil.isNotNull(order.getPayFee())){
//                    payFee = payFee.add(order.getPayFee());
//                }
//                order.setPayFee(payFee);
//                /** 更新订单信息 */
//                int i = orderService.updateRlOrderByOrderNo(order);
//                if(i == 0){
//                    throw new ServiceException("【改价】更新订单信息失败");
//                }
//                /** 记录订单履历 */
//                if(!orderOperService.recordOrderHistory(order.getOrderNo(),ServiceConstants.ORDER_OPERATION_CHANGE_PRICE,
//                        etOrder1.getStatus(),order.getStatus(),etOrder1.getPayFee(),order.getPayFee(),etOrder1.getUserId(),etOrder1.getPhone(),
//                        "订单改价,修改前价格："+etOrder1.getPayFee()+",修改后价格："+order.getPayFee())){
//                    throw new ServiceException("【改价】更新订单信息失败");
//                }
//            }
//            return Boolean.TRUE;
//        });
//        if(!execute)throw new ServiceException("【改价】失败");
//        return toAjax(1);
//    }


    /**
     * 根据sn查询正在进行中的订单
     */
    @GetMapping("/getInProgressOrder")
    public AjaxResult getInProgressOrder(String sn)
    {
        logger.info("【根据sn查询正在进行中的订单】：sn=【{}】", sn);
        if(StrUtil.isBlank(sn)){
            throw new ServiceException("【根据sn查询正在进行中的订单】sn不能为空");
        }
        OrderVO order = orderService.getInProgressOrder(sn);
        return success(order);
    }


    /**
     * 新增反馈意见
     */
    @Log(title = "反馈意见", businessType = BusinessType.INSERT)
    @PostMapping("/feedback")
    public AjaxResult add(@RequestBody Feedback asFeedback)
    {
        logger.info("【新增反馈意见】：【{}】", JSON.toJSONString(asFeedback));
        //校验请求参数
        if(StrUtil.isBlank(asFeedback.getType())){
            return error("类型不能为空");
        }
        if(StrUtil.isBlank(asFeedback.getIssueDescription())){
            return error("问题描述不能为空");
        }
        if(StrUtil.isBlank(asFeedback.getContactInfo())){
            return error("联系方式不能为空");
        }
        return toAjax(feedbackService.insertAsFeedback(asFeedback));
    }

    /**
     * 管理员提现
     */
    @Log(title = "管理员提现", businessType = BusinessType.ADMINWITHDRAW)
    @PostMapping("/admin/withdraw")
    public AjaxResult add(@RequestBody WithdrawQuery withdraw)
    {
        logger.info("管理员提现请求：【{}】", JSON.toJSON(withdraw));
        int i = userService.adminWithdraw(withdraw);
        return toAjax(i);
    }

    /**
     * 修改手机号
     */
    @Log(title = "修改手机号", businessType = BusinessType.ADMINWITHDRAW)
    @PostMapping("/edit/phone")
    public AjaxResult editPhone(String phone)
    {
        logger.info("修改手机号：【phone={}】", phone);
        User updateUser = new User();
        updateUser.setUserId(getUserId());
        updateUser.setPhonenumber(phone);
        int i = userService.updateUser(updateUser);
        if(i == 0) return AjaxResult.warn("修改手机号失败");
        return toAjax(i);
    }


//    /**
//     *  根据token获取提现渠道的列表
//     */
//    @GetMapping("/getUserWithdrawChannelList")
//    public AjaxResult getUserWithdrawChannelList()
//    {
//        logger.info("根据token获取提现渠道列表：【userId="+getUserId()+"】");
//        return AjaxResult.success(userWithdrawService.selectRlUserWithdrawListByUserId(getUserId()));
//    }

    /**
     * 查询全局提现渠道列表
     */
    @GetMapping("/withdrawList")
    public AjaxResult withdrawList(ChannelWithdrawQuery query)
    {
        List<ChannelWithdrawVO> list = etChannelWithdrawService.selectChannelWithdrawList(query);
        return success(list);
    }

    /**
     * 获取字典类型列表
     */
    @GetMapping("/getDictData")
    public AjaxResult getDictData(String dictType)
    {
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType(dictType);
        List<SysDictData> list = dictDataService.selectDictDataList(sysDictData);
        return success(list);
    }

//    /**
//     * 查询收费模板列表
//     */
//    @GetMapping("/ruleList")
//    public AjaxResult ruleList(EFeeRule rlFeeRule)
//    {
//        List<EFeeRule> list = feeRuleService.selectRlFeeRuleList(rlFeeRule);
//        return success(list);
//    }

    /**
     * 新增收费模板
     */
    @Log(title = "收费模板", businessType = BusinessType.INSERT)
    @PostMapping("/rule")
    public AjaxResult addRule(@RequestBody FeeRule rlFeeRule)
    {
        logger.info("新增收费模板：【{}】", JSON.toJSONString(rlFeeRule));
        return toAjax(feeRuleService.insertEFeeRule(rlFeeRule));
    }

    /**
     * 修改收费模板
     */
    @Log(title = "收费模板", businessType = BusinessType.UPDATE)
    @PutMapping("/rule")
    public AjaxResult editRule(@RequestBody FeeRule rlFeeRule)
    {
        logger.info("修改收费模板：【{}】", JSON.toJSONString(rlFeeRule));
        return toAjax(feeRuleService.updateEFeeRule(rlFeeRule));
    }

    /**
     * 删除收费模板(逻辑删除)
     */
    @Log(title = "收费模板", businessType = BusinessType.DELETE)
    @DeleteMapping("/rule/{ruleIds}")
    public AjaxResult removeRule(@PathVariable Long[] ruleIds)
    {
        logger.info("删除收费模板：【{}】", JSON.toJSONString(ruleIds));
        for (Long ruleId : ruleIds){
            FeeRule eFeeRule = new FeeRule();
            eFeeRule.setDelFlag("1");
            eFeeRule.setRuleId(ruleId);
            int i = feeRuleService.updateEFeeRule(eFeeRule);
            if(i == 0){
                return AjaxResult.warn("【收费模板】删除失败");
            }
        }
        return toAjax(1);
    }

//    /**
//     * 查询车辆型号列表
//     */
//    @GetMapping("/modelList")
//    public AjaxResult modelList(EModelQuery eModel)
//    {
//        logger.info("查询车辆型号列表：【{}】", JSON.toJSONString(eModel));
//        List<EModelVO> list = modelService.selectEModelList(eModel);
//        return success(list);
//    }
//
//    /**
//     * 新增车辆型号
//     */
//    @Log(title = "车辆型号", businessType = BusinessType.INSERT)
//    @PostMapping("/model")
//    public AjaxResult addModel(@RequestBody EModel eModel)
//    {
//        logger.info("新增车辆型号：【{}】", JSON.toJSONString(eModel));
//        return toAjax(modelService.insertEModel(eModel));
//    }

//    /**
//     * 修改车辆型号
//     */
//    @Log(title = "车辆型号", businessType = BusinessType.UPDATE)
//    @PutMapping("/model")
//    public AjaxResult editModel(@RequestBody EModel etModel)
//    {
//        logger.info("修改车辆型号：【{}】", JSON.toJSONString(etModel));
//        return toAjax(modelService.updateEModel(etModel));
//    }
//
//    /**
//     * 删除车辆型号
//     */
//    @Log(title = "车辆型号", businessType = BusinessType.DELETE)
//    @DeleteMapping("/model/{modelIds}")
//    public AjaxResult removeModel(@PathVariable Long[] modelIds)
//    {
//        logger.info("删除车辆型号：【{}】", JSON.toJSONString(modelIds));
//        for (Long modelId : modelIds){
//            EModel model = new EModel();
//            model.setDelFlag("1");
//            model.setModelId(modelId);
//            int i = modelService.updateEModel(model);
//            if(i == 0){
//                return AjaxResult.warn("【车辆型号】删除失败");
//            }
//        }
//        return toAjax(1);
//    }

    /**
     * 查询文章列表
     */
    @GetMapping("/article/list")
    public TableDataInfo articleList(Article etArticle)
    {
        startPage();
        List<Article> list = articleService.selectAsArticleListByApp(etArticle);
        return getDataTable(list);
    }

    /**
     * 获取文章详细信息
     */
    @GetMapping(value = "/article/{articleId}")
    public AjaxResult getArticleInfo(@PathVariable("articleId") Long articleId)
    {
        return success(articleService.selectAsArticleByArticleId(articleId));
    }

    /**
     * 根据查询文章分类列表
     */
    @GetMapping("/classify/list")
    public AjaxResult classifyList(ArticleClassify asArticleClassify)
    {
        logger.info("查询分类列表：【asArticleClassify="+asArticleClassify+"】");
        List<ArticleClassify> asArticleClassifies = articleClassifyService.selectClassifyList(asArticleClassify);
        return success(asArticleClassifies);
    }

}
