package com.ruoyi.ss.article.service;

import com.ruoyi.ss.article.domain.Article;

import java.util.List;

/**
 * 文章Service接口
 *
 * @author qiuzhenzhao
 * @date 2023-12-06
 */
public interface IArticleService
{
    /**
     * 查询文章
     *
     * @param articleId 文章主键
     * @return 文章
     */
    public Article selectAsArticleByArticleId(Long articleId);

    /**
     * 查询文章列表
     *
     * @param etArticle 文章
     * @return 文章集合
     */
    public List<Article> selectAsArticleList(Article etArticle);

    /**
     * 查询文章列表
     *
     * @param etArticle 文章
     * @return 文章集合
     */
    public List<Article> selectAsArticleListByApp(Article etArticle);


    /**
     * 查询文章列表（带数据隔离）
     *
     * @param etArticle 文章
     * @return 文章集合
     */
    public List<Article> selectAsArticleListByIsolate(Article etArticle);


    /**
     * 新增文章
     *
     * @param etArticle 文章
     * @return 结果
     */
    public int insertAsArticle(Article etArticle);

    /**
     * 修改文章
     *
     * @param etArticle 文章
     * @return 结果
     */
    public int updateAsArticle(Article etArticle);

    /**
     * 批量删除文章
     *
     * @param articleIds 需要删除的文章主键集合
     * @return 结果
     */
    public int deleteAsArticleByArticleIds(Long[] articleIds);

    /**
     * 删除文章信息
     *
     * @param articleId 文章主键
     * @return 结果
     */
    public int deleteAsArticleByArticleId(Long articleId);

    /**
     * 根据文章id获取标签列表
     *
     * @param articleId 文章主键
     * @return 结果
     */
    public String[] getTagList(Long articleId);

//    /**
//     * 获取动态分类列表
//     *
//     * @param
//     * @return 结果
//     */
//    public List<DynamicClassify> selectDynamicClassifyList();
}
