package com.ruoyi.ss.city.service.impl;

import com.ruoyi.ss.city.domain.CityCode;
import com.ruoyi.ss.city.mapper.CityCodeMapper;
import com.ruoyi.ss.city.service.ICityCodeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 城市区号Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class CityCodeServiceImpl implements ICityCodeService
{
    @Resource
    private CityCodeMapper rlCityCodeMapper;

    /**
     * 查询城市区号
     *
     * @param code 城市区号
     * @return 城市区号
     */
    @Override
    public CityCode selectRlCityCodeByCode(String code)
    {
        return rlCityCodeMapper.selectRlCityCodeByCode(code);
    }


    /**
     * 查询城市区号列表
     *
     * @param rlCityCode 城市区号
     * @return 城市区号
     */
    @Override
    public List<CityCode> selectRlCityCodeList(CityCode rlCityCode)
    {
        return rlCityCodeMapper.selectRlCityCodeList(rlCityCode);
    }

    /**
     * 新增城市区号
     *
     * @param rlCityCode 城市区号
     * @return 结果
     */
    @Override
    public int insertRlCityCode(CityCode rlCityCode)
    {
        return rlCityCodeMapper.insertRlCityCode(rlCityCode);
    }

    /**
     * 修改城市区号
     *
     * @param rlCityCode 城市区号
     * @return 结果
     */
    @Override
    public int updateRlCityCode(CityCode rlCityCode)
    {
        return rlCityCodeMapper.updateRlCityCode(rlCityCode);
    }

    /**
     * 批量删除城市区号
     *
     * @param codeIds 需要删除的城市区号主键
     * @return 结果
     */
    @Override
    public int deleteRlCityCodeByCodeIds(Long[] codeIds)
    {
        return rlCityCodeMapper.deleteRlCityCodeByCodeIds(codeIds);
    }

    /**
     * 删除城市区号信息
     *
     * @param codeId 城市区号主键
     * @return 结果
     */
    @Override
    public int deleteRlCityCodeByCodeId(Long codeId)
    {
        return rlCityCodeMapper.deleteRlCityCodeByCodeId(codeId);
    }
}
