package com.ruoyi.ss.feeRule.service.impl;

import java.util.Collections;
import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.feeRule.domain.FeeRule;
import com.ruoyi.ss.feeRule.domain.FeeRuleVO;
import com.ruoyi.ss.feeRule.mapper.FeeRuleMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.feeRule.domain.FeeRuleQuery;
import com.ruoyi.ss.feeRule.service.IFeeRuleService;

import static com.ruoyi.common.constant.ServiceConstants.CHARGE_MODE_HOURLY;

/**
 * 收费模板Service业务层处理
 *
 * @author qzz
 * @date 2024-11-20
 */
@Service
public class FeeRuleServiceImpl implements IFeeRuleService
{
    @Autowired
    private FeeRuleMapper eFeeRuleMapper;

    /**
     * 查询收费模板
     *
     * @param ruleId 收费模板主键
     * @return 收费模板
     */
    @Override
    public FeeRuleVO selectEFeeRuleByRuleId(Long ruleId)
    {
        return eFeeRuleMapper.selectEFeeRuleByRuleId(ruleId);
    }

    /**
     * 查询收费模板列表
     *
     * @param eFeeRule 收费模板
     * @return 收费模板
     */
    @Override
    public List<FeeRuleVO> selectEFeeRuleList(FeeRuleQuery eFeeRule)
    {
        return eFeeRuleMapper.selectEFeeRuleList(eFeeRule);
    }

    /**
     * 新增收费模板
     *
     * @param eFeeRule 收费模板
     * @return 结果
     */
    @Override
    public int insertEFeeRule(FeeRule eFeeRule)
    {
        eFeeRule.setCreateTime(DateUtils.getNowDate());
        verifyHourlyUnique(eFeeRule);
        return eFeeRuleMapper.insertEFeeRule(eFeeRule);
    }

    private void verifyHourlyUnique(FeeRule eFeeRule) {
        if (eFeeRule.getMode().equals(CHARGE_MODE_HOURLY)){
            int i = eFeeRuleMapper.verifyHourlyUnique(eFeeRule.getRoomId());
            ServiceUtil.assertion(i > 0, "该房间已存在小时计费模板");
        }
    }

    /**
     * 修改收费模板
     *
     * @param eFeeRule 收费模板
     * @return 结果
     */
    @Override
    public int updateEFeeRule(FeeRule eFeeRule)
    {
        verifyHourlyUnique(eFeeRule);
        return eFeeRuleMapper.updateEFeeRule(eFeeRule);
    }

    /**
     * 批量删除收费模板
     *
     * @param ruleIds 需要删除的收费模板主键
     * @return 结果
     */
    @Override
    public int deleteEFeeRuleByRuleIds(Long[] ruleIds)
    {
        return eFeeRuleMapper.deleteEFeeRuleByRuleIds(ruleIds);
    }

    /**
     * 删除收费模板信息
     *
     * @param ruleId 收费模板主键
     * @return 结果
     */
    @Override
    public int deleteEFeeRuleByRuleId(Long ruleId)
    {
        return eFeeRuleMapper.deleteEFeeRuleByRuleId(ruleId);
    }

    /**
     * 根据房间id获取套餐
     */
    @Override
    public List<FeeRuleVO> selectEFeeRuleListByRoomId(Long roomId) {
        return eFeeRuleMapper.selectEFeeRuleListByRoomId(roomId);
    }

    @Override
    public FeeRule selectEveryHoursFeeRuleByRoomId(Long roomId) {
        return null;
    }

    @Override
    public List<FeeRuleVO> selectCustomFeeRuleByRoomId(Long roomId) {
        return Collections.emptyList();
    }
}
