package com.ruoyi.ss.order.service.impl;

import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.UserVO;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.order.domain.dto.ReOrderDTO;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.room.domain.RoomVO;
import com.ruoyi.ss.feeRule.domain.FeeRuleVO;
import com.ruoyi.ss.store.domain.StoreVO;
import com.ruoyi.ss.order.domain.OrderBO;
import com.ruoyi.ss.order.domain.dto.OrderDTO;
import com.ruoyi.ss.order.service.IOrderConverter;
import com.ruoyi.ss.feeRule.service.IFeeRuleService;
import com.ruoyi.ss.room.service.IRoomService;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.user.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
public class OrderConverterImpl implements IOrderConverter {


    @Resource
    private IRoomService roomService;

    @Resource
    private IFeeRuleService feeRuleService;

    @Resource
    private IUserService userService;

    @Resource
    private IStoreService storeService;

    @Autowired
    private IOrderService orderService;

    @Autowired
    private DeviceService deviceService;


    // 订单dto转bo
    @Override
    public OrderBO toOrderBO(OrderDTO order) {

        // 房间
        RoomVO eRoomVO = roomService.selectERoomByRoomId(order.getRoomId());
        ServiceUtil.assertion(eRoomVO == null, "房间不存在");

        // 房间门
        DeviceVO door = deviceService.selectById(eRoomVO.getDoorId());
        ServiceUtil.assertion(door == null, "房间门设备不存在");

        // 店铺
        StoreVO eStoreVO = storeService.selectEStoreByStoreId(eRoomVO.getStoreId());
        ServiceUtil.assertion(eStoreVO == null, "店铺不存在");

        // 大门
        DeviceVO gate = deviceService.selectById(eStoreVO.getGateId());
        ServiceUtil.assertion(gate == null, "店铺大门设备不存在");

        // 商户
        UserVO merchantVO =  userService.selectUserById(eStoreVO.getManagerId());
        ServiceUtil.assertion(merchantVO == null, "商户不存在");

        ServiceUtil.assertion(merchantVO.getServiceFeeProportion() == null, "商户【"+merchantVO.getUserName()+"】的平台服务费未配置");
        ServiceUtil.assertion(merchantVO.getPayChannel() == null, "商户【"+merchantVO.getUserName()+"】的支付渠道未配置");

        // 套餐
        FeeRuleVO eFeeRuleVO = feeRuleService.selectEFeeRuleByRuleId(order.getRuleId());
        ServiceUtil.assertion(eFeeRuleVO == null, "套餐不存在");

        // 用户
        UserVO eUserVO = userService.selectUserById(SecurityUtils.getUserId());
        ServiceUtil.assertion(eUserVO == null, "用户不存在");
        ServiceUtil.assertion(eUserVO.getWxopenid() == null, "用户微信openId为空");

        OrderBO orderBO = new OrderBO();
        orderBO.setParams(order);
        orderBO.setRoom(eRoomVO);
        orderBO.setDoor(door);
        orderBO.setFeeRule(eFeeRuleVO);
        orderBO.setMerchant(merchantVO);
        orderBO.setStore(eStoreVO);
        orderBO.setGate(gate);
        orderBO.setUser(eUserVO);
        orderBO.setType(ServiceConstants.ORDER_TYPE_RESERVE);
        return orderBO;
    }

    // 订单dto转bo
    @Override
    public OrderBO toOrderBO(ReOrderDTO order) {

        // 原订单
        OrderVO originalOrder = orderService.selectRlOrderByOrderNo(order.getOriginalOrderNo());

        // 房间
        RoomVO eRoomVO = roomService.selectERoomByRoomId(originalOrder.getRoomId());
        ServiceUtil.assertion(eRoomVO == null, "房间不存在");

        // 房间门
        DeviceVO door = deviceService.selectById(eRoomVO.getDoorId());
        ServiceUtil.assertion(door == null, "房间门设备不存在");

        // 店铺
        StoreVO eStoreVO = storeService.selectEStoreByStoreId(eRoomVO.getStoreId());
        ServiceUtil.assertion(eStoreVO == null, "店铺不存在");

        // 大门
        DeviceVO gate = deviceService.selectById(eStoreVO.getGateId());
        ServiceUtil.assertion(gate == null, "店铺大门设备不存在");

        // 商户
        UserVO merchantVO =  userService.selectUserById(eStoreVO.getManagerId());
        ServiceUtil.assertion(merchantVO == null, "商户不存在");

        ServiceUtil.assertion(merchantVO.getServiceFeeProportion() == null, "商户【"+merchantVO.getUserName()+"】的平台服务费未配置");
        ServiceUtil.assertion(merchantVO.getPayChannel() == null, "商户【"+merchantVO.getUserName()+"】的支付渠道未配置");

        // 套餐
        FeeRuleVO eFeeRuleVO = feeRuleService.selectEFeeRuleByRuleId(order.getRuleId());
        ServiceUtil.assertion(eFeeRuleVO == null, "套餐不存在");

        // 用户
        UserVO eUserVO = userService.selectUserById(SecurityUtils.getUserId());
        ServiceUtil.assertion(eUserVO == null, "用户不存在");
        ServiceUtil.assertion(eUserVO.getWxopenid() == null, "用户微信openId为空");

        OrderBO orderBO = new OrderBO();
        orderBO.setReOrderDTO(order);
        orderBO.setRoom(eRoomVO);
        orderBO.setDoor(door);
        orderBO.setFeeRule(eFeeRuleVO);
        orderBO.setMerchant(merchantVO);
        orderBO.setStore(eStoreVO);
        orderBO.setGate(gate);
        orderBO.setUser(eUserVO);
        orderBO.setOriginalOrder(originalOrder);
        orderBO.setType(ServiceConstants.ORDER_TYPE_REORDER);
        return orderBO;
    }
}
