package com.ruoyi.ss.order.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.User;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.service.BonusConverter;
import com.ruoyi.ss.bonus.service.BonusService;
import com.ruoyi.ss.changeBalance.service.IChangeBalanceService;
import com.ruoyi.ss.feeRule.domain.FeeRuleVO;
import com.ruoyi.ss.feeRule.service.IFeeRuleService;
import com.ruoyi.ss.msgLog.service.IMsgLogService;
import com.ruoyi.ss.order.domain.dto.ReOrderDTO;
import com.ruoyi.ss.order.domain.vo.ReservedTimePeriod;
import com.ruoyi.ss.order.mapper.OrderMapper;
import com.ruoyi.ss.order.domain.Order;
import com.ruoyi.ss.order.domain.OrderBO;
import com.ruoyi.ss.order.domain.OrderQuery;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.orderOper.domain.OrderOper;
import com.ruoyi.ss.orderOper.service.IOrderOperService;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.domain.RefundVO;
import com.ruoyi.ss.user.service.IUserService;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import com.ruoyi.ss.order.domain.dto.OrderDTO;
import com.ruoyi.ss.order.service.IOrderConverter;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.order.service.IOrderValidator;
import com.ruoyi.ss.refund.service.IRefundService;
import com.ruoyi.ss.room.service.IRoomService;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;

/**
 * 订单Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class OrderServiceImpl implements IOrderService
{

    private static final Logger logger = LoggerFactory.getLogger(OrderServiceImpl.class);

    @Resource
    private OrderMapper orderMapper;

    @Resource
    private IWxPayService wxPayService;

    @Autowired
    private IFeeRuleService feeRuleService;


    @Resource
    private IOrderConverter orderConverter;

    @Resource
    private IOrderValidator orderValidator;

    @Autowired
    private ISysConfigService configService;

    @Resource
    private IUserService userService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IRefundService refundService;

    @Autowired
    private IOrderOperService rlOrderOperService;

    @Autowired
    private IOrderOperService orderOperService;

    @Autowired
    private IRoomService roomService;

    @Autowired
    private BonusConverter bonusConverter;

    @Autowired
    private BonusService bonusService;

    @Autowired
    private IChangeBalanceService changeBalanceService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IMsgLogService msgLogService;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode3}")
    private String templateCode3;

    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    @Override
    public OrderVO selectRlOrderByOrderId(Long orderId)
    {
        return orderMapper.selectRlOrderByOrderId(orderId);
    }

    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public OrderVO selectRlOrderByOrderNo(String orderNo)
    {
        return orderMapper.selectRlOrderByOrderNo(orderNo);
    }

//    /**
//     * 第三方订单号
//     *
//     * @param outTradeNo 订单号
//     * @return 订单
//     */
//    @Override
//    public OrderVO selectRlOrderByOutTradeNo(String outTradeNo)
//    {
//        return orderMapper.selectRlOrderByOutTradeNo(outTradeNo);
//    }


    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public OrderVO getOrderInfoByOrderNo(String orderNo)
    {
        OrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(order)){
            throw new ServiceException("订单不存在");
        }
        /** 超时规则*/
        Long ruleId = order.getRuleId();
        FeeRuleVO feeRule = feeRuleService.selectEFeeRuleByRuleId(ruleId);
        order.setRule(feeRule);

        /** 操作记录*/
        OrderOper orderOper = new OrderOper();
        orderOper.setOrderNo(orderNo);
        List<OrderOper> rlOrderOpers = rlOrderOperService.selectRlOrderOperList(orderOper);
        order.setOrderOpers(rlOrderOpers);
        /** 收益明细*/
//        List<RlDividendDetailVO> rlDividendDetails = rlDividendDetailService.selectRlDividendDetailListByOrderNo(orderNo);
//        order.setRlDividendDetails(rlDividendDetails);
//
//        RlDeliveryOrder deliveryOrder = deliveryOrderService.selectRlDeliveryOrderByOrderNo(orderNo);
//        if(ObjectUtil.isNotNull(deliveryOrder)){
//            order.setDeliveryStatus(deliveryOrder.getStatus());
//            order.setDeliveryOrder(deliveryOrder);
//        }

        /** 显示可退款金额  根据订单号查询可退款金额  refundableAmount*/
        getRefundable(orderNo, order);
        return order;
    }

    private void getRefundable(String orderNo, OrderVO order) {
        RefundVO refundVO = refundService.getRefundedAmount(orderNo);
        BigDecimal refundableAmount = order.getPayFee().subtract(refundVO.getTotalRefundAmount());
//        BigDecimal refundableDispatchFee = order.getDispatchFee().subtract(refundVO.getTotalDispatchFee());
//        BigDecimal refundableDeliveryFee = order.getDeliveryFee().subtract(refundVO.getTotalDeliveryFee());
//        BigDecimal refundableDeposit = order.getDeposit().subtract(refundVO.getTotalDeposit());
//        BigDecimal refundableLeaseFee = order.getLeaseFee().subtract(refundVO.getTotalLeaseFee());
        order.setRefundableTotalAmount(refundableAmount);
//        order.setRefundableDispatchFee(refundableDispatchFee);
//        order.setRefundableDeliveryFee(refundableDeliveryFee);
//        order.setRefundableDeposit(refundableDeposit);
//        order.setRefundableLeaseFee(refundableLeaseFee);
    }

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    @DataScope(merchantAlias = "m", userAlias = "u")
    public List<OrderVO> selectRlOrderList(OrderQuery rlOrder)
    {
        return orderMapper.selectRlOrderList(rlOrder);
    }

    /**
     * 查询订单列表  不待数据隔离
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<OrderVO> selectRlOrderListNoIsolate(OrderQuery rlOrder)
    {
        return orderMapper.selectRlOrderList(rlOrder);
    }

    /**
     * 查询所有订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<OrderVO> selectAllRlOrderList(OrderQuery rlOrder)
    {
        return orderMapper.selectAllRlOrderList(rlOrder);
    }

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int insertRlOrder(OrderQuery rlOrder)
    {
        rlOrder.setCreateTime(DateUtils.getNowDate());
        return orderMapper.insertRlOrder(rlOrder);
    }

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrder(Order rlOrder)
    {
        return orderMapper.updateRlOrder(rlOrder);
    }

    /**
     * 根据订单号修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrderByOrderNo(Order rlOrder)
    {
        return orderMapper.updateRlOrderByOrderNo(rlOrder);
    }


    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderIds(Long[] orderIds)
    {
        return orderMapper.deleteRlOrderByOrderIds(orderIds);
    }

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderId(Long orderId)
    {
        return orderMapper.deleteRlOrderByOrderId(orderId);
    }

    /**
     * 下单
     * 1. 创建订单
     * 2. 预支付
     * 3.
     *
     * @param order 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO createOrder(OrderDTO order) {
        OrderBO orderBO = orderConverter.toOrderBO(order);
        return getPrepayResponseVO(orderBO);
    }

    /**
     * 续单
     *
     * @param reOrder 续单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO reOrder(ReOrderDTO reOrder) {
        OrderBO orderBO = orderConverter.toOrderBO(reOrder);
        return getPrepayResponseVO(orderBO);
    }

    /**
     *  确认支付
     * @param orderNo 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO confirmPay(String orderNo) {
        OrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_PAID)){
            throw new ServiceException("订单已支付");
        }
        if(!order.getStatus().equals(ServiceConstants.ORDER_STATUS_TO_BE_PAID)){
            throw new ServiceException("订单非待支付状态");
        }
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayResponseVO responseVO = new PrepayResponseVO();
        responseVO.setResponse(response);
        responseVO.setOrderNo(orderNo);
        return responseVO;
    }

    @NotNull
    private PrepayResponseVO getPrepayResponseVO(OrderBO orderBO) {

        // 生成订单
        Order order = parseToOrder(orderBO);

        // 生成分成列表
        List<Bonus> bonusList = bonusConverter.toPo(orderBO);
        orderBO.setBonusList(bonusList);

//        BigDecimal serviceFee = orderBO.getMerchant().getServiceFeeProportion()
//                .multiply(order.getPayFee())
//                .setScale(2, RoundingMode.HALF_UP);// 服务费 = 48 * 0.05 = 2.4
//        order.setPlatformServiceFee(serviceFee);
//        logger.info("计算出平台服务费：{}",serviceFee);

        PrepayResponseVO responseVO = new PrepayResponseVO();
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【下单】==================");
            /** 保存订单*/
            int i = orderMapper.insertRlOrder(order);
            ServiceUtil.assertion(i < 1, "下单失败：保存订单失败");
            /** 支付参数 */
            PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
            responseVO.setResponse(response);
            responseVO.setOrderNo(order.getOrderNo());

            for (Bonus bonus : orderBO.getBonusList()) {
                bonus.setBillId(order.getOrderId());
                bonus.setBillNo(order.getOrderNo());
            }
            int bonusInsert = bonusService.batchInsert(orderBO.getBonusList());
            ServiceUtil.assertion(bonusInsert != orderBO.getBonusList().size(), "创建分成失败");

            /** 发短信给代理商*/
//            YPMsgUtils.sendPlaceOrderMsg(orderNo,rlAgentVO.getPhone(),order.getPhone());
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("【下单】失败");
        return responseVO;
    }

    // 转换为订单所需的数据
    private Order parseToOrder(OrderBO orderBO) {
        // 校验
        ServiceUtil.assertion(orderValidator.preAddOrder(orderBO));

        Order order = new Order();
        String orderNo ;
        Date reserveStartTime;
        String payType;
        if(orderBO.getType().equals(ServiceConstants.ORDER_TYPE_RESERVE)){
            orderNo = IdUtils.getOrderNo("yd");
            reserveStartTime = orderBO.getParams().getReserveStartTime();
            payType = orderBO.getParams().getPayType();
        }else{
            orderNo = IdUtils.getOrderNo("xd");
            reserveStartTime = orderBO.getOriginalOrder().getReserveStartTime();
            payType = orderBO.getReOrderDTO().getPayType();
        }

        FeeRuleVO feeRule = orderBO.getFeeRule();

        //套餐
        order.setRuleId(feeRule.getRuleId());
        order.setMode(feeRule.getMode());
        order.setHours(feeRule.getHours());
        order.setPrice(feeRule.getPrice());
        order.setExplain(feeRule.getExplain());

        /** 如果是预定订单，根据预约开始时间计算出预约结束时间
         * 如果是续费订单，预约开始时间是原订单的预约结束时间 */
        // dto
        order.setReserveStartTime(reserveStartTime);
        order.setReserveEndTime(DateUtils.getTimeAfterXUnit(reserveStartTime,feeRule.getHours(),"hours"));
        order.setDuration(DateUtils.timeDifferenceInHours(reserveStartTime, order.getReserveEndTime()));

        //房间
        order.setRoomId(orderBO.getRoom().getRoomId());
        order.setRoomName(orderBO.getRoom().getRoomName());
        order.setRoomType(orderBO.getRoom().getType());
        order.setRoomMac(orderBO.getDoor().getMac());
        order.setRoomMac2(orderBO.getDoor().getMac2());
        order.setRoomSn(orderBO.getDoor().getSn());

        //基本信息
        order.setCreateTime(DateUtils.getNowDate());
        order.setOrderNo(orderNo);
        order.setPayType(payType);
        order.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        order.setType(ServiceConstants.ORDER_TYPE_RESERVE);
        order.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_PAID);

        //用户
        order.setUserId(orderBO.getUser().getUserId());
        order.setUserName(orderBO.getUser().getUserName());
        order.setPhone(orderBO.getUser().getPhonenumber());
        order.setRealName(orderBO.getUser().getRealName());

        // 商户
        order.setMerchantId(orderBO.getMerchant().getUserId());
        order.setMerchantName(orderBO.getMerchant().getUserName());

        // 店铺
        order.setStoreId(orderBO.getStore().getStoreId());
        order.setStoreName(orderBO.getStore().getName());
        order.setCityId(orderBO.getStore().getCityId());
        order.setGateMac(orderBO.getGate().getMac());
        order.setGateMac2(orderBO.getGate().getMac2());
        order.setGateSn(orderBO.getGate().getSn());
        order.setWifi(orderBO.getStore().getWifi());
        order.setWifiPassword(orderBO.getStore().getWifiPassword());

        // 价格
        /** 根据套餐计算价格 */
        order.setTotalFee(feeRule.getPrice().multiply(new BigDecimal(feeRule.getHours())));
        order.setPayFee(order.getTotalFee());

        // 取消时间
        String configByKey = configService.selectConfigByKey("rl.order.auto.cancel");
        order.setAutoCancelTime(DateUtils.getTimeAfterXMinutes(order.getCreateTime(),Integer.parseInt(configByKey)));

        return order;
    }

    /**
     * 扫码绑定车辆
     */
    @Override
    public int bandDevice(String orderNo, String sn) {
        OrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        OrderVO order1 = getInProgressOrder(sn);
        if(ObjectUtil.isNotNull(order1)){
            throw new RuntimeException("该车辆【"+sn+"】有进行中的订单：");
        }
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【扫码绑定车辆】==================");
            Order rlOrder = new Order();
            rlOrder.setOrderNo(orderNo);
            rlOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
//            rlOrder.setPickupTime(DateUtils.getNowDate());
//            Date expiryTime = DateUtils.getTimeAfterXUnit(DateUtils.getNowDate(), order.getNum(), order.getRentalUnit());
//            rlOrder.setExpiryTime(expiryTime);
            int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
            if(i > 0){
                /** 更新设备状态 */
//                EDeviceQuery rlDevice = new EDeviceQuery();
//                rlDevice.setSn(sn);
//                rlDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
//                deviceService.updateEDeviceBySn(rlDevice);
            }
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("扫码绑定车辆失败");
        return 1;
    }

    /**
     * 用户扫码绑定车辆
     *  1. 校验sn是否存在
     *  2. 绑定车辆,在车辆中添加用户id
     */
    @Override
    public int userBandDevice(Long userId, String sn) {
//        EDevice device = deviceService.selectEDeviceBySn(sn);
//        if(device == null){
//            throw new ServiceException("设备不存在:"+sn);
//        }
//        if(device.getMerchantId() != null && device.getMerchantId() != 0){
//            throw new ServiceException("该车辆已绑定用户");
//        }
//        Boolean execute = transactionTemplate.execute(e -> {
//            logger.info("=================【用户扫码绑定车辆】==================");
//            EDeviceQuery rlDevice = new EDeviceQuery();
//            rlDevice.setSn(sn);
//            rlDevice.setMerchantId(userId);
//            deviceService.updateEDeviceBySn(rlDevice);
//            return Boolean.TRUE;
//        });
//        if(Boolean.FALSE.equals(execute))throw new ServiceException("【用户扫码绑定车辆】失败");
        return 1;
    }

//        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
//        if(order == null){
//            throw new RuntimeException("订单不存在");
//        }
//        if(StrUtil.isNotBlank(order.getSn())){
//            throw new RuntimeException("该订单已绑定车辆："+order.getSn());
//        }
//        RlOrderVO order1 = getInProgressOrder(sn);
//        if(ObjectUtil.isNotNull(order1)){
//            throw new RuntimeException("该车辆【"+sn+"】有进行中的订单：");
//        }
//        Boolean execute = transactionTemplate.execute(e -> {
//            logger.info("=================【扫码绑定车辆】==================");
//            RlOrder rlOrder = new RlOrder();
//            rlOrder.setOrderNo(orderNo);
//            rlOrder.setSn(sn);
//            rlOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
//            rlOrder.setPickupTime(DateUtils.getNowDate());
//            Date expiryTime = DateUtils.getTimeAfterXUnit(DateUtils.getNowDate(), order.getNum(), order.getRentalUnit());
//            rlOrder.setExpiryTime(expiryTime);
//            int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
//            if(i > 0){
//                /** 更新设备状态 */
//                EDevice rlDevice = new EDevice();
//                rlDevice.setSn(sn);
//                rlDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
//                deviceService.updateEDeviceBySn(rlDevice);
//            }
//            return Boolean.TRUE;
//        });
//        if(!execute)throw new ServiceException("扫码绑定车辆失败");
//        return 1;
//    }

    /**
     * 续单
     * 到期时间
     * 1.  根据原来的订单号，写入新的订单
     * 2.  续单订单标记类型
     * 3.
     */
    @Override
    public PrepayResponseVO relet(OrderQuery query) {
        OrderVO order = orderMapper.selectRlOrderByOrderNo(query.getOrderNo());
        if(ObjectUtil.isNull(order)){
            throw new RuntimeException("订单【"+query.getOrderNo()+"】不存在");
        }
        OrderQuery orderQuery = new OrderQuery();
        BeanUtils.copyBeanProp(orderQuery,order);
        /** 原订单已经到期  续单订单的到期时间是根据原订单的到期时间算出来的 */
//        Date nowDate = DateUtils.getNowDate();
//        if (order.getExpiryTime().before(nowDate)) {
//            /** 到期后先计算逾期费用，保存在订单中，在还车时一起结算 */
//            EOrder updateOrder = new EOrder();
//            updateOrder.setOrderId(order.getOrderId());
//            BigDecimal overdueFee= BigDecimal.ZERO;
//            if(ObjectUtil.isNull(order.getExpiryTime())){
//                throw new ServiceException("【续单】订单没有到期时间");
//            }
////            if (order.getExpiryTime().before(DateUtils.getNowDate())) {
////                overdueFee = deviceService.computeOverdueFee(order, updateOrder);
////            }
//            updateOrder.setOverdueFee(overdueFee);
//            int i = updateRlOrder(updateOrder);
//            if(i==0){
//                throw new ServiceException("更新订单状态失败");
//            }
//        }
        orderQuery.setOriginalOrderNo(query.getOrderNo());
        orderQuery.setRuleId(query.getRuleId());
        String newOrderNo = IdUtils.getOrderNo("xz");
//        getPrepayResponseVO(orderQuery,newOrderNo,ServiceConstants.ORDER_TYPE_RELET,SecurityUtils.getUserId())
        return null;
    }

    /**
     * 订单取消
     *
     */
    @Override
    public Boolean orderCancel(String orderNo) {
        if(StrUtil.isBlank(orderNo)){
            throw new RuntimeException("订单号不能为空");
        }
        OrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        // todo  刷新支付结果
        // 取消订单改成配送工单还没有接单前可以取消订单
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【订单取消】==================");
            if(ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
                // 如果订单已支付则发起退款
                OrderQuery rlOrderQuery = new OrderQuery();
                BeanUtils.copyBeanProp(rlOrderQuery,order);
                int refund = refund(rlOrderQuery);
                if(refund <= 0){
                    throw new RuntimeException("【订单取消】退款失败");
                }
            }

            /** 更新订单状态 */
            Order rlOrder = new Order();
            rlOrder.setOrderNo(orderNo);
            rlOrder.setStatus(ServiceConstants.ORDER_STATUS_USER_CANCE);
            int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
            ServiceUtil.assertion(i != 1, "取消订单失败：更新订单状态失败");

            /** 更新支付订单  关闭所有的支付订单 */



            /** 记录订单履历 */
            boolean b = orderOperService.recordOrderHistory(orderNo, ServiceConstants.ORDER_OPERATION_USER_CANCEL,
                    order.getStatus(), rlOrder.getStatus(), order.getPayFee(), order.getPayFee(), order.getUserId(), order.getPhone(), "取消订单");
            ServiceUtil.assertion(!b, "取消订单失败：记录订单履历失败");
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("订单取消失败");
        return true;
    }

    /**
     * 车损扣款
     */
    @Override
    public Boolean deductMoney(BigDecimal money, String orderNo) {
        Order rlOrder = new Order();
        rlOrder.setOrderNo(orderNo);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        /** 记录订单履历 */
        OrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
//        if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_DEDUCTION,
//                order.getStatus(),order.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),
//                "车损扣款：扣款金额："+money+"，实际押金退款："+order.getDeposit()+"-"+money)){
//            throw new ServiceException("【车损扣款】更新订单信息失败");
//        }
        return i>0;
    }

//    /**
//     * 商家还车
//     *
//     */
//    @Override
//    public Boolean merchantReturn(String orderNo) {
//        EOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
//        if(ObjectUtil.isNull(order)){
//            throw new ServiceException("订单不存在");
//        }
////        BigDecimal deposit = order.getDeposit();
////        BigDecimal overdueFee = order.getOverdueFee();
////        BigDecimal deductionAmount = order.getDeductionAmount();
////        BigDecimal remainingDeposit = deposit.subtract(overdueFee).subtract(deductionAmount);
//        if(!ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
//          throw new ServiceException("订单未支付，不能还车", HttpStatus.ERROR);
//        }
//        if(!(ServiceConstants.ORDER_STATUS_TO_BE_AUDITED.equals(order.getStatus()) || ServiceConstants.ORDER_STATUS_IN_USE.equals(order.getStatus()))){
//            throw new ServiceException("订单非待审核或骑行中状态，不能还车", HttpStatus.ERROR);
//        }
//        Boolean execute = transactionTemplate.execute(e -> {
//            /** 1.更新订单信息*/
//            updateOrderStatus(orderNo);
//            /** 2.更新车辆状态*/
////            updateDeviceInfo(order.getSn());
//            /** 3.记录订单履历*/
//            if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_RETURN_END,
//                    order.getStatus(),order.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"已退款：-"+remainingDeposit+"元")){
//                throw new ServiceException("【记录订单履历失败】");
//            }
//            if(remainingDeposit.compareTo(BigDecimal.ZERO) > 0){
//                /** 4.执行退还剩余押金操作*/
//                /** 5.新增一条退款记录*/
//                String outRefundNo = saveDepositRefundObj(order, remainingDeposit);
//                logger.info("【商家还车】退还剩余押金："+remainingDeposit);
//                wxPayService.refund(order, "审核后退押金",remainingDeposit,outRefundNo);
//            }
//            return Boolean.TRUE;
//        });
//        if(!execute)throw new ServiceException("商家还车失败");
//        return true;
//    }

    @NotNull
    private String saveDepositRefundObj(OrderVO rlOrderVO, BigDecimal refundAmount) {
        String outRefundNo = IdUtils.getOrderNo("ref");
        Refund refund1= createRefund("押金退款", rlOrderVO.getUserId(), rlOrderVO.getOrderNo(), refundAmount, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO, outRefundNo,ServiceConstants.REFUND_TYPE_DEPOSIT);
        int refundResult = refundService.insertEtRefund(refund1);
        if(refundResult>0){
            logger.info("保存退款对象成功");
        }
//        logger.info("总押金：【{}】，退款金额：【{}】", rlOrderVO.getDeposit(), refundAmount);
        return outRefundNo;
    }

    private void updateOrderStatus(String orderNo) {
        Order updateOrder = new Order();
        updateOrder.setOrderNo(orderNo);
//        updateOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        int i = orderMapper.updateRlOrderByOrderNo(updateOrder);
        if(i==0){
            logger.info("【商家还车】更新订单信息失败");
            throw new ServiceException("【商家还车】更新订单信息失败");
        }
    }

    /**
     * 退款
     * 1. 校验
     * 2. 用户余额不足判断
     * 3. 计算出退款的比例
     * 4. 遍历用户，新增账变
     * 5. 新增退款明细
     * 6. 发起退款
     */
    @Transactional
    @Override
    public int refund(OrderQuery etOrder) {
//        //根据订单号查询订单信息
//        /** 1. 校验 */
//        EOrderVO rlOrderVO = orderMapper.selectRlOrderByOrderNo(etOrder.getOrderNo());
//        refundVerify(rlOrderVO);
//        /** 2.计算出退款的比例 保留两位小数点  比如退款金额为5元，  订单金额为32元  退款比例 = 5/32 = 0.16 （四舍五入）由系统来平账*/
//        //退款金额  调度费(商家取车)  配送费（送车上门）  租赁费
////        BigDecimal refundAmount = new BigDecimal("0");
////        BigDecimal refundPercentage = refundAmount.divide(rlOrderVO.getPayFee(), 2, RoundingMode.HALF_UP);
////        BigDecimal finalRefundAmount = refundAmount;
//        Boolean execute = transactionTemplate.execute(e -> {
//            /** 3. 根据分成列表，遍历用户，新增账变 */
////            refundDividendHandle(rlOrderVO, refundPercentage);
//            /** 4.记录退款表  创建退款对象*/
//            String outRefundNo = saveRefundObj(rlOrderVO, finalRefundAmount, dispatchFee, deliveryFee, leaseFee);
//            if(!orderOperService.recordOrderHistory(rlOrderVO.getOrderNo(),ServiceConstants.ORDER_OPERATION_REFUND,
//                    rlOrderVO.getStatus(),rlOrderVO.getStatus(),rlOrderVO.getPayFee(),rlOrderVO.getPayFee(),rlOrderVO.getUserId(),rlOrderVO.getPhone(),"已退款: "+finalRefundAmount+"元")){
//                throw new ServiceException("【退款】更新订单信息失败");
//            }
//            /** 5. 发起退款 */
//            wxPayService.refund(rlOrderVO, "商家退款金额", finalRefundAmount,outRefundNo);
//            return Boolean.TRUE;
//        });
//        if(!execute)throw new ServiceException("【订单退款】失败");
        return 1;
    }

    @NotNull
    private String saveRefundObj(OrderVO rlOrderVO, BigDecimal refundAmount, BigDecimal dispatchFee, BigDecimal deliveryFee, BigDecimal leaseFee) {
        String outRefundNo = IdUtils.getOrderNo("ref");
        Refund refund1= createRefund("商家退款金额", rlOrderVO.getUserId(), rlOrderVO.getOrderNo(), refundAmount, dispatchFee, deliveryFee, leaseFee, outRefundNo,ServiceConstants.REFUND_TYPE_SYSTEM);
        int refundResult = refundService.insertEtRefund(refund1);
        if(refundResult>0){
            logger.info("保存退款对象成功");
        }
        logger.info("总金额：【{}】，退款金额：【{}】", rlOrderVO.getTotalFee(), refundAmount);
        return outRefundNo;
    }

//    private void refundDividendHandle(RlOrderVO rlOrderVO, BigDecimal refundPercentage) {
//        List<RlDividendDetailVO> dividendDetails = rlDividendDetailService.selectRlDividendDetailListByOrderNo(rlOrderVO.getOrderNo());
//        for(RlDividendDetail detail:dividendDetails){
//            /** 4.1 余额不足判断 */
////            RlUserExt rlUserExt = userExtService.selectRlUserExtByUserId(detail.getPartnerId());
//            EUser user = userService.selectUserById(rlUserExt.getUserId());
//            // 计算出这个用户将要退款的金额
//            BigDecimal userRefundAmount = detail.getDividendAmount().multiply(refundPercentage);//用户退款金额  = 0.16 * 5元 = 0.8元  如果这个值小于余额则提示 余额不足
//            BigDecimal subtract = user.getBalance().subtract(userRefundAmount);
//            if(subtract.compareTo(BigDecimal.ZERO) <= 0){
//                throw new ServiceException("用户【"+user.getPhonenumber()+"】余额不足，不能退款");
//            }
//            /** 4.2 新增账变 */
//            int i = changeBalanceService.generateChanggeBalance(rlOrderVO.getOrderNo(), rlOrderVO.getOutTradeNo(), ServiceConstants.FLOW_TYPE_DISBURSE,
//                    ServiceConstants.ORDER_REFUND, userRefundAmount, rlUserExt.getUserId(), user.getUserName(), user.getPhonenumber(),user.getUserType());
//            if(i<=0)throw new ServiceException("用户【"+user.getPhonenumber()+"】新增账变失败");
//            /** todo 4.3 更新分成表中的退款金额 */
//            int i1 = rlDividendDetailService.updateRefundAmount(detail.getId(), userRefundAmount);
//            if(i1<=0)throw new ServiceException("【退款】更新分成表中的退款金额失败");
//        }
//    }

    /** 退款校验*/
    private void refundVerify(OrderVO rlOrderVO) {
        if(ObjectUtil.isNull(rlOrderVO)){
            throw new ServiceException("订单不存在");
        }
//        if (!ServiceConstants.ORDER_STATUS_ORDER_END.equals(rlOrderVO.getStatus())) {
//            throw new ServiceException("退款失败，订单未结束");
//        }
        if(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT.equals(rlOrderVO.getPaid())){
            throw new ServiceException("订单未支付，不能退款");
        }
    }

    @Override
    public Refund createRefund(String reason, Long userId, String orderNo, BigDecimal refundAmount, BigDecimal dispatchFee, BigDecimal deliveryFee, BigDecimal leaseFee, String outRefundNo, String type) {
        Refund etRefund = new Refund();
        etRefund.setAmount(refundAmount);
        etRefund.setReason(reason);
        etRefund.setDispatchFee(dispatchFee);
        etRefund.setDeliveryFee(deliveryFee);
        etRefund.setLeaseFee(leaseFee);
        etRefund.setOrderNo(orderNo);
        etRefund.setUserId(userId);
        etRefund.setRefundNo(outRefundNo);
        etRefund.setType(type);
        if(type.equals(ServiceConstants.REFUND_TYPE_DEPOSIT)){
            etRefund.setItemDesc("押金退款");
        }else{
            StringBuilder itemDesc = new StringBuilder();
            if(ObjectUtil.isNotNull(deliveryFee) && !deliveryFee.equals(BigDecimal.ZERO)){
                itemDesc.append("配送费:").append(deliveryFee).append("元,");
            }
            if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
                itemDesc.append("调度费:").append(dispatchFee).append("元,");
            }
            if(ObjectUtil.isNotNull(leaseFee) && !leaseFee.equals(BigDecimal.ZERO)){
                itemDesc.append("租赁费:").append(leaseFee).append("元,");
            }
            etRefund.setItemDesc(itemDesc.toString());
        }
        return etRefund;
    }

//
//    /**
//     * 计算价格
//     */
//    @Override
//    public PriceVO calculatePrice(EOrderQuery order) {
//        PriceVO priceVO = new PriceVO();
//        BigDecimal totalFee = BigDecimal.ZERO;
//        // 租金
//        BigDecimal rentFee = rentFee(order, priceVO);
//        totalFee = totalFee.add(rentFee);
//        priceVO.setTotalFee(totalFee);
//        return priceVO;
//    }

//    /**
//     * 根据订单号查询可还车的店铺
//     */
//    @Override
//    public List<StoreVo> getReturnStoreList(String orderNo,String lon,String lat) {
//        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
//        List<StoreVo> storeVos = storeService.selectSmStoreListBYMerchantId(order.getMerchantId());
//        for (StoreVo storeVo:storeVos) {
//            BigDecimal storeLon = storeVo.getLng();
//            BigDecimal storeLat = storeVo.getLat();
//            if (storeLon != null && storeLat != null && StrUtil.isNotBlank(lon) && StrUtil.isNotBlank(lat)) {
//                double[] point1 = {storeLon.doubleValue(), storeLat.doubleValue()};
//                double[] point2 = {Double.valueOf(lon), Double.valueOf(lat)};
//                double distance = GeoUtils.haversineDistance(point1, point2);
//                BigDecimal distanceFormatted = new BigDecimal(distance).setScale(1, RoundingMode.HALF_UP);
//                storeVo.setDistance(distanceFormatted.doubleValue());
//            }
//        }
//        List<StoreVo> storeList = new ArrayList<>(storeVos);
//        Collections.sort(storeList, new Comparator<StoreVo>() {
//            @Override
//            public int compare(StoreVo o1, StoreVo o2) {
//                // 注意：由于我们想要按距离由近到远排序，因此当 o1 的距离小于 o2 的距离时，应返回 -1
//                // 当距离相等时返回 0，当 o1 的距离大于 o2 的距离时返回 1
//                return Double.compare(o1.getDistance(), o2.getDistance());
//            }
//        });
//        return storeList;
//    }

    /**
     * 根据sn查询正在进行中的订单
     */
    @Override
    public OrderVO getInProgressOrder(String sn) {
        List<OrderVO> orders = orderMapper.getInProgressOrder(sn);
        if(ObjectUtil.isNotNull(orders.size()) && !orders.isEmpty()){
            return orders.get(0);
        }
        return null;
    }

    @Override
    public void validate(OrderQuery orderQuery) {
        if(StrUtil.isBlank(orderQuery.getOrderNo())){
            throw new ServiceException("订单号不能为空");
        }
        if(StrUtil.isBlank(orderQuery.getType())){
            throw new ServiceException("类型不能为空");
        }
        if(orderQuery.getRuleId() == null || orderQuery.getRuleId() <= 0){
            throw new ServiceException("套餐id不能为空");
        }
    }

    /**
     * 首页统计
     */
    @Override
    public IndexAdminVo indexStatistics(String startTime,String endTime,Long userId) {
        // 定义统一的日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        // 根据用户id查询代理商，根据代理商做数据权限隔离
        User rlUser = userService.selectUserById(userId);
        if(!ServiceConstants.USER_TYPE_MERCHANT.equals(rlUser.getUserType())){
            throw new ServiceException("您没有权限访问");
        }
        return getIndexAdminVoByAgentId(startTime, endTime, formatter,rlUser.getUserId());
    }

    /**
     * 已售数量
     */
    @Override
    public int selectSoldNum(Long roomId) {
        return orderMapper.selectSoldNum(roomId);
    }

    /**
     * 已预约订单时间段
     */
    @Override
    public List<ReservedTimePeriod> selectReservedTimes(Long roomId, Date startTime, Date endTime) {
        return orderMapper.selectReservedTimes(roomId, startTime, endTime);
    }

    /**
     * 更换包间
     */
    @Override
    public int changeRoom(OrderQuery order) {
        return orderMapper.changeRoom(order);
    }

    /**
     * 时间段内是否有预定单
     */
    @Override
    public int isReserved(Long roomId, Date startTime, Date endTime) {
        return orderMapper.isReserved(roomId, startTime, endTime);
    }

    @NotNull
    private IndexAdminVo getIndexAdminVoByAgentId(String startTime, String endTime, DateTimeFormatter formatter,Long merchantId) {
        // 校验输入时间格式并判断时间区间是否超过15天
        if (StrUtil.isBlank(startTime) && StrUtil.isBlank(endTime)) {
            // 获取今日订单数量和收入
            startTime = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate());
            endTime = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate());
        }
        // 使用LocalDate统一日期处理
        LocalDate startDate = LocalDate.parse(startTime, formatter);
        LocalDate endDate = LocalDate.parse(endTime, formatter);

        // 计算两者之间的天数差异
        long daysBetween = ChronoUnit.DAYS.between(startDate, endDate);
        if (daysBetween > 15) {
            throw new ServiceException("时间范围不能超过15天");
        }

        // 创建结果对象
        IndexAdminVo indexAdminVo = new IndexAdminVo();
        ArrayList<IndexAdminVo.IncomeVo> incomeVos = new ArrayList<>();

        // 遍历从 startDate 到 endDate 的每一天，生成订单数据
        for (LocalDate date = startDate; !date.isAfter(endDate); date = date.plusDays(1)) {
            IndexAdminVo.IncomeVo orderFee = new IndexAdminVo.IncomeVo();
            String day = date.format(formatter);
            String startDateStr = day + " "+ Constants.DATE_FORMAT_START_PEREND;
            String endDateStr = day + " " +Constants.DATE_FORMAT_END_PEREND;
            orderFee.setDay(day); // 设置每一天的日期
            BigDecimal orderFee1 = orderMapper.getOrderFee(startDateStr, endDateStr, merchantId);//订单金额
            orderFee.setOrderFee(orderFee1);
            orderFee.setOrderNum(orderMapper.getOrderNum(startDateStr, endDateStr, merchantId));//订单数量

            incomeVos.add(orderFee);
        }
        // 设置收入数据
        indexAdminVo.setIncomeVoList(incomeVos);
        // 获取今日订单数量和收入
        String todayStart = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate()) + " " + Constants.DATE_FORMAT_START_PEREND;
        String todayEnd = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate()) + " " + Constants.DATE_FORMAT_END_PEREND;

        // 查询今日的订单数和营收
        Integer todayOrderNum = orderMapper.getOrderNum(todayStart, todayEnd, merchantId);
        BigDecimal todayIncome = orderMapper.getOrderFee(todayStart, todayEnd, merchantId);

        // 设置今日的订单数和营收
        indexAdminVo.setTodayOrderNum(todayOrderNum);
        indexAdminVo.setTodayIncome(todayIncome);

        String yesterdayStart = DateUtils.getYYYY_MM_DD(DateUtils.addDays(DateUtils.getNowDate(), -1)) + " " + Constants.DATE_FORMAT_START_PEREND;
        String yesterdayEnd = DateUtils.getYYYY_MM_DD(DateUtils.addDays(DateUtils.getNowDate(), -1)) + " " + Constants.DATE_FORMAT_END_PEREND;
        // 查询昨日的订单数和营收
        Integer yesterdayOrderNum = orderMapper.getOrderNum(yesterdayStart, yesterdayEnd, merchantId);
        BigDecimal yesterdayIncome = orderMapper.getOrderFee(yesterdayStart, yesterdayEnd, merchantId);

        // 计算较昨日的营收和订单数
        BigDecimal comparedYesterdayIncome = todayIncome.subtract(yesterdayIncome);
        BigDecimal comparedYesterdayOrderNum = new BigDecimal(todayOrderNum - yesterdayOrderNum);

        // 设置较昨日的营收和订单数
        indexAdminVo.setComparedYesterdayIncome(comparedYesterdayIncome);
        indexAdminVo.setComparedYesterdayOrderNum(comparedYesterdayOrderNum);

        // 设置车辆统计数据
        IndexAdminVo.DeviceVO deviceVO = new IndexAdminVo.DeviceVO();
//        deviceVO.setTotal(deviceService.getTatalDeviceCount(merchantId,null,null));
//        deviceVO.setIdle(deviceService.getTatalDeviceCount(merchantId,ServiceConstants.VEHICLE_STATUS_NORMAL,null));
        String[] statusList = new String[] { ServiceConstants.VEHICLE_STATUS_IN_USING,ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK };
//        deviceVO.setRent(deviceService.getTatalDeviceCount(merchantId,null,statusList));
        indexAdminVo.setDeviceVO(deviceVO);
        return indexAdminVo;
    }




}
