package com.ruoyi.ss.refund.service.impl;

import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.order.domain.Order;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.domain.RefundVO;
import com.ruoyi.ss.refund.mapper.RefundMapper;
import com.ruoyi.ss.refund.service.IRefundService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 退款订单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
@Service
public class RefundServiceImpl implements IRefundService
{
    @Resource
    private RefundMapper etRefundMapper;

    /**
     * 查询退款订单
     *
     * @param id 退款订单主键
     * @return 退款订单
     */
    @Override
    public Refund selectEtRefundById(Long id)
    {
        return etRefundMapper.selectEtRefundById(id);
    }

    /**
     * 根据订单号查询退款订单对象
     *
     * @param orderNo 订单号
     * @return 退款订单
     */
    @Override
    public List<Refund> selectEtRefundByOrderNo(String orderNo)
    {
        return etRefundMapper.selectEtRefundByOrderNo(orderNo);
    }

    /**
     * 根据退款单号查询退款订单对象
     *
     * @param refundNo 退款订号
     * @return 退款订单
     */
    @Override
    public Refund selectEtRefundByRefundNo(String refundNo)
    {
        return etRefundMapper.selectEtRefundByRefundNo(refundNo);
    }


    /**
     * 查询退款订单列表
     *
     * @param rlRefund 退款订单
     * @return 退款订单
     */
    @Override
    public List<Refund> selectEtRefundList(Refund rlRefund)
    {
        return etRefundMapper.selectEtRefundList(rlRefund);
    }

    /**
     * 查询退款订单列表
     *
     * @param rlRefund 退款订单
     * @return 退款订单
     */
    @Override
    @DataScope(merchantAlias = "m", userAlias = "u")
    public List<Refund> selectEtRefundListWithIsolate(Refund rlRefund)
    {
        return etRefundMapper.selectEtRefundList(rlRefund);
    }

    /**
     * 新增退款订单
     *
     * @param rlRefund 退款订单
     * @return 结果
     */
    @Override
    public int insertEtRefund(Refund rlRefund)
    {
        rlRefund.setCreateTime(DateUtils.getNowDate());
        return etRefundMapper.insertEtRefund(rlRefund);
    }

    /**
     * 修改退款订单
     *
     * @param rlRefund 退款订单
     * @return 结果
     */
    @Override
    public int updateEtRefund(Refund rlRefund)
    {
        return etRefundMapper.updateEtRefund(rlRefund);
    }

    /**
     * 批量删除退款订单
     *
     * @param ids 需要删除的退款订单主键
     * @return 结果
     */
    @Override
    public int deleteEtRefundByIds(Long[] ids)
    {
        return etRefundMapper.deleteEtRefundByIds(ids);
    }

    /**
     * 删除退款订单信息
     *
     * @param id 退款订单主键
     * @return 结果
     */
    @Override
    public int deleteEtRefundById(Long id)
    {
        return etRefundMapper.deleteEtRefundById(id);
    }

    /**
     * 根据退款单号修改退款订单
     * @param rlRefund
     * @return
     */
    @Override
    public int updateEtRefundByRefundNo(Refund rlRefund) {
        return etRefundMapper.updateEtRefundByRefundNo(rlRefund);
    }

    /**
     * 判断是否用户是否还有未退款的押金
     *
     * @param userId 用户id
     * @return 结果
     */
    @Override
    public List<Order> checkUserDeposit(Long userId) {
       return etRefundMapper.checkUserDeposit(userId);
    }

    /**
     * 统计已退款金额
     *
     * @param orderNo 订单号
     * @return 结果
     */
    @Override
    public RefundVO getRefundedAmount(String orderNo) {
        return etRefundMapper.getRefundedAmount(orderNo);
    }
}
