package com.ruoyi.ss.room.service.impl;

import java.util.List;

import com.ruoyi.ss.device.service.DeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.room.mapper.RoomMapper;
import com.ruoyi.ss.room.domain.Room;
import com.ruoyi.ss.room.domain.RoomVO;
import com.ruoyi.ss.room.domain.RoomQuery;
import com.ruoyi.ss.room.service.IRoomService;

/**
 * 房间Service业务层处理
 *
 * @author qzz
 * @date 2024-11-20
 */
@Service
public class RoomServiceImpl implements IRoomService
{
    @Autowired
    private RoomMapper eRoomMapper;

    @Autowired
    private DeviceService smDeviceService;

    /**
     * 查询房间
     *
     * @param roomId 房间主键
     * @return 房间
     */
    @Override
    public RoomVO selectERoomByRoomId(Long roomId)
    {
        return eRoomMapper.selectERoomByRoomId(roomId);
    }

    /**
     * 查询房间列表
     *
     * @param eRoom 房间
     * @return 房间
     */
    @Override
    public List<RoomVO> selectERoomList(RoomQuery eRoom)
    {
        return eRoomMapper.selectERoomList(eRoom);
    }

    /**
     * 根据店铺id查询房间列表
     *
     * @param storeId 店铺id
     * @return 房间集合
     */
    @Override
    public List<RoomVO> selectERoomListByStoreId(Long storeId) {
        return eRoomMapper.selectERoomListByStoreId(storeId);
    }

    /**
     * 新增房间
     *
     * @param eRoom 房间
     * @return 结果
     */
    @Override
    public int insertERoom(Room eRoom)
    {
        return eRoomMapper.insertERoom(eRoom);
    }

    /**
     * 修改房间
     *
     * @param eRoom 房间
     * @return 结果
     */
    @Override
    public int updateERoom(Room eRoom)
    {
        return eRoomMapper.updateERoom(eRoom);
    }

    /**
     * 批量删除房间
     *
     * @param roomIds 需要删除的房间主键
     * @return 结果
     */
    @Override
    public int deleteERoomByRoomIds(Long[] roomIds)
    {
        return eRoomMapper.deleteERoomByRoomIds(roomIds);
    }

    /**
     * 删除房间信息
     *
     * @param roomId 房间主键
     * @return 结果
     */
    @Override
    public int deleteERoomByRoomId(Long roomId)
    {
        return eRoomMapper.deleteERoomByRoomId(roomId);
    }

    /**
     * 开门
     * @param roomId 房间id
     */
    @Override
    public boolean openGate(Long roomId) {
        RoomVO eRoomVO = eRoomMapper.selectERoomByRoomId(roomId);
        return smDeviceService.switchDevice(eRoomVO.getDoorId(), true);
    }
}
